@@ -58,6 +58,8 @@ import nom.tam.fits.LongValueException;
 import nom.tam.fits.TruncatedFileException;
 import nom.tam.fits.header.Standard;
 import nom.tam.fits.header.hierarch.BlanksDotHierarchKeyFormatter;
+import nom.tam.fits.header.hierarch.IHierarchKeyFormatter;
+import nom.tam.fits.header.hierarch.StandardIHierarchKeyFormatter;
 import nom.tam.util.AsciiFuncs;
 import nom.tam.util.ComplexValue;
 import nom.tam.util.Cursor;
@@ -532,11 +534,11 @@ public class HeaderCardTest {
         // now test a longString
         FitsFactory.setLongStringsEnabled(true);
 
-        hc = new HeaderCard(key, "a verly long value that must be splitted over multiple lines to fit the card",
+        hc = new HeaderCard(key, "a very long value that must be splitted over multiple lines to fit the card",
                 "the comment is also not the smallest");
 
-        assertEquals("HIERARCH TEST1 TEST2 INT = 'a verly long value that must be splitted over mult&'" + //
-                "CONTINUE  'iple lines to fit the card' / the comment is also not the smallest   ", hc.toString());
+        assertEquals("HIERARCH TEST1 TEST2 INT = 'a very long value that must be splitted over multi&'" + //
+                "CONTINUE  'ple lines to fit the card' / the comment is also not the smallest    ", hc.toString());
 
     }
 
@@ -605,10 +607,10 @@ public class HeaderCardTest {
         FitsFactory.setSkipBlankAfterAssign(true);
         String key = "HIERARCH.TEST1.TEST2.INT";
 
-        HeaderCard hc = new HeaderCard(key, "a verly long value that must be splitted over multiple lines to fit the card",
+        HeaderCard hc = new HeaderCard(key, "a very long value that must be splitted over multiple lines to fit the card",
                 "the comment is also not the smallest");
 
-        assertEquals("HIERARCH TEST1 TEST2 INT ='a verly long value that must be splitted over multi&'" + //
+        assertEquals("HIERARCH TEST1 TEST2 INT = 'a very long value that must be splitted over multi&'" + //
                 "CONTINUE  'ple lines to fit the card' / the comment is also not the smallest    ", hc.toString());
 
     }
@@ -687,43 +689,7 @@ public class HeaderCardTest {
     }
 
     @Test(expected = LongValueException.class)
-    public void testRewriteLongStringOverflow1() {
-        FitsFactory.setUseHierarch(true);
-        FitsFactory.setLongStringsEnabled(false);
-        // We can read this card, even though the keyword is longer than allowed due to the missing
-        // spaces around the '='.
-        HeaderCard hc = HeaderCard
-                .create("HIERARCH ZZZZZZZZZZZZZZZZZZZZZZZZZZZZZZZZZZZZZZZZZZZZZZZZZZZZZZZZZZZZZZZZZZZZ=''");
-        // But we should not be able to re-write this card.
-        hc.toString();
-    }
-
-    @Test(expected = LongValueException.class)
-    public void testRewriteLongStringOverflow2() {
-        FitsFactory.setUseHierarch(true);
-        FitsFactory.setLongStringsEnabled(false);
-        // We can read this card, even though the keyword is longer than allowed due to the missing
-        // spaces around the '='.
-        HeaderCard hc = HeaderCard
-                .create("HIERARCH ZZZZZZZZZZZZZZZZZZZZZZZZZZZZZZZZZZZZZZZZZZZZZZZZZZZZZZZZZZZZZZZZZZZZZ=T");
-        // But we should not be able to re-write this card.
-        hc.toString();
-    }
-
-    @Test(expected = LongValueException.class)
-    public void testRewriteLongStringOverflow3() {
-        FitsFactory.setUseHierarch(true);
-        FitsFactory.setLongStringsEnabled(false);
-        // We can read this card, even though the keyword is longer than allowed due to the missing
-        // spaces around the '='.
-        HeaderCard hc = HeaderCard
-                .create("HIERARCH ZZZZZZZZZZZZZZZZZZZZZZZZZZZZZZZZZZZZZZZZZZZZZZZZZZZZZZZZZZZZZZZZZZZZZZ=");
-        // But we should not be able to re-write this card.
-        hc.toString();
-    }
-
-    @Test(expected = LongValueException.class)
-    public void testRewriteLongStringOverflow4() {
+    public void testRewriteLongBooleanHierarchExcept() {
         FitsFactory.setUseHierarch(true);
         FitsFactory.setLongStringsEnabled(false);
         // We can read this card, even though the keyword is longer than allowed due to the missing
@@ -734,16 +700,47 @@ public class HeaderCardTest {
         hc.setValue(true);
     }
 
-    @Test(expected = LongValueException.class)
-    public void testRewriteLongStringOverflow5() {
+    @Test(expected = IllegalStateException.class)
+    public void testRewriteStringHierarchExcept() {
+        FitsFactory.setUseHierarch(true);
+        FitsFactory.setLongStringsEnabled(false);
+        // We can read this card, even though the keyword is longer than allowed due to the missing
+        // spaces around the '='.
+        HeaderCard hc = HeaderCard
+                .create("HIERARCH ZZZZZZZZZZZZZZZZZZZZZZZZZZZZZZZZZZZZZZZZZZZZZZZZZZZZZZZZZZZZZZZZZZZZZZ=");
+        // But we should not be able to set even an empty string value..
+        hc.setValue("");
+    }
+
+    public void testRewriteEmptyStringHierarch() {
+        FitsFactory.setUseHierarch(true);
+        FitsFactory.setLongStringsEnabled(false);
+        // We can read this card, even though the keyword is longer than allowed due to the missing
+        // spaces around the '='.
+        HeaderCard hc = HeaderCard.create("HIERARCH ZZZZZZZZZZZZZZZZZZZZZZZZZZZZZZZZZZZZZZZZZZZZZZZZZZZZZZZZZZZZZZZZZZZZ=");
+        // We should not be able to set an empty string value...
+        hc.setValue("");
+    }
+
+    public void testRewriteEmptyLongStringHierarch() {
         FitsFactory.setUseHierarch(true);
         FitsFactory.setLongStringsEnabled(true);
         // We can read this card, even though the keyword is longer than allowed due to the missing
         // spaces around the '='.
-        HeaderCard hc = HeaderCard.create("HIERARCH ZZZZZZZZZZZZZZZZZZZZZZZZZZZZZZZZZZZZZZZZZZZZZZZZZZZZZZZZZZZZZZZZZZZ='&'"
-                + "CONTINUE 'continues here' / comment");
-        // But we should not be able to re-write this card.
-        hc.toString();
+        HeaderCard hc = HeaderCard.create("HIERARCH ZZZZZZZZZZZZZZZZZZZZZZZZZZZZZZZZZZZZZZZZZZZZZZZZZZZZZZZZZZZZZZZZZZZZ=");
+        // We should not be able to set an empty string value...
+        hc.setValue("");
+    }
+
+    @Test(expected = IllegalStateException.class)
+    public void testRewriteLongStringHierarchExcept() {
+        FitsFactory.setUseHierarch(true);
+        FitsFactory.setLongStringsEnabled(true);
+        // We can read this card, even though the keyword is longer than allowed due to the missing
+        // spaces around the '='.
+        HeaderCard hc = HeaderCard.create("HIERARCH ZZZZZZZZZZZZZZZZZZZZZZZZZZZZZZZZZZZZZZZZZZZZZZZZZZZZZZZZZZZZZZZZZZZZ=");
+        // But we should not be able to set a non-empty string value...
+        hc.setValue("boo!");
     }
 
     @Test
@@ -1104,21 +1101,21 @@ public class HeaderCardTest {
         FitsFactory.setUseHierarch(true);
         HeaderCard headerCard = new HeaderCard("HIERARCH.TEST1.TEST2.TEST3.TEST4.TEST5.TEST6", "xy", null);
         assertEquals("HIERARCH.TEST1.TEST2.TEST3.TEST4.TEST5.TEST6", headerCard.getKey());
-        assertEquals("HIERARCH TEST1 TEST2 TEST3 TEST4 TEST5 TEST6 ='xy'                              ",
+        assertEquals("HIERARCH TEST1 TEST2 TEST3 TEST4 TEST5 TEST6 = 'xy'                             ",
                 headerCard.toString());
         assertEquals("HIERARCH.TEST1.TEST2.TEST3.TEST4.TEST5.TEST6",
                 new HeaderCard(headerCardToStream(headerCard)).getKey());
 
         FitsFactory.setHierarchFormater(new BlanksDotHierarchKeyFormatter(1));
         assertEquals("HIERARCH.TEST1.TEST2.TEST3.TEST4.TEST5.TEST6", headerCard.getKey());
-        assertEquals("HIERARCH TEST1.TEST2.TEST3.TEST4.TEST5.TEST6 ='xy'                              ",
+        assertEquals("HIERARCH TEST1.TEST2.TEST3.TEST4.TEST5.TEST6 = 'xy'                             ",
                 headerCard.toString());
         assertEquals("HIERARCH.TEST1.TEST2.TEST3.TEST4.TEST5.TEST6",
                 new HeaderCard(headerCardToStream(headerCard)).getKey());
 
         FitsFactory.setHierarchFormater(new BlanksDotHierarchKeyFormatter(2));
         assertEquals("HIERARCH.TEST1.TEST2.TEST3.TEST4.TEST5.TEST6", headerCard.getKey());
-        assertEquals("HIERARCH  TEST1.TEST2.TEST3.TEST4.TEST5.TEST6 ='xy'                             ",
+        assertEquals("HIERARCH  TEST1.TEST2.TEST3.TEST4.TEST5.TEST6 = 'xy'                            ",
                 headerCard.toString());
         assertEquals("HIERARCH.TEST1.TEST2.TEST3.TEST4.TEST5.TEST6",
                 new HeaderCard(headerCardToStream(headerCard)).getKey());
@@ -1694,13 +1691,13 @@ public class HeaderCardTest {
         assertEquals("UTC", card.getValue());
         assertEquals("All dates are in UTC time", card.getComment());
         assertEquals("HIERARCH.TIMESYS.BBBB.CCCC", card.getKey());
-        assertEquals("HIERARCH TIMESYS BBBB CCCC ='UTC' / All dates are in UTC time                   ", card.toString());
+        assertEquals("HIERARCH TIMESYS BBBB CCCC = 'UTC' / All dates are in UTC time                  ", card.toString());
 
         card = HeaderCard.create("HIERARCH TIMESYS.BBBB.CCCC ='UTC ' / All dates are in UTC time");
         assertEquals("UTC", card.getValue());
         assertEquals("All dates are in UTC time", card.getComment());
         assertEquals("HIERARCH.TIMESYS.BBBB.CCCC", card.getKey());
-        assertEquals("HIERARCH TIMESYS BBBB CCCC ='UTC' / All dates are in UTC time                   ", card.toString());
+        assertEquals("HIERARCH TIMESYS BBBB CCCC = 'UTC' / All dates are in UTC time                  ", card.toString());
     }
 
     private String makeTestString(int n) {
@@ -1946,4 +1943,92 @@ public class HeaderCardTest {
         }
     }
 
+    @Test
+    public void testEmptyHierarchKey() {
+        IHierarchKeyFormatter fmt = new StandardIHierarchKeyFormatter();
+        Assert.assertEquals("", fmt.toHeaderString(". ."));
+    }
+
+    @Test
+    public void testHierarchSpace() {
+        HeaderCard c = HeaderCard
+                .create("HIERARCH.AAA.BBB.CCC.DDD.EEE.FFF.GGG.HHH.III.JJJ.KKK.LLL.MMM.NNN.OOO.PPP.QQQ.R=T");
+        Assert.assertEquals(1, c.spaceForValue());
+
+        c = HeaderCard.create("HIERARCH.AAA.BBB.CCC.DDD.EEE.FFF.GGG.HHH.III.JJJ.KKK.LLL.MMM.NNN.OOO.PPP.QQQ='x'");
+        Assert.assertEquals(3, c.spaceForValue());
+    }
+
+    @Test
+    public void testSetLongEmptyString() {
+        FitsFactory.setLongStringsEnabled(true);
+        HeaderCard c = new HeaderCard("TEST", "blah", "empty string");
+        c.setValue("");
+        Assert.assertEquals("", c.getValue());
+    }
+
+    @Test
+    public void testChangeKeyLongString() {
+        FitsFactory.setLongStringsEnabled(true);
+        HeaderCard c = new HeaderCard("TEST", "blah", "empty string");
+
+        c.changeKey("TEST2");
+        Assert.assertEquals("TEST2", c.getKey());
+
+        FitsFactory.setLongStringsEnabled(false);
+        c.changeKey("TEST3");
+        Assert.assertEquals("TEST3", c.getKey());
+    }
+
+    @Test
+    public void testChangeKeyEmptyLongString() {
+        FitsFactory.setLongStringsEnabled(true);
+        HeaderCard c = new HeaderCard("TEST", "", "empty string");
+
+        c.changeKey("TEST2");
+        Assert.assertEquals("TEST2", c.getKey());
+    }
+
+    @Test(expected = LongValueException.class)
+    public void testChangeKeyLongStringExcept() {
+        FitsFactory.setLongStringsEnabled(true);
+        HeaderCard c = new HeaderCard("TEST", "abc", "no comment");
+        // No space for '&'...
+        c.changeKey("HIERARCH.TEST.AAA.BBB.CCC.DDD.EEE.FFF.GGG.HHH.III.JJJ.KKK.LLL.MMM.NNN.OOO.PPP");
+    }
+
+    @Test(expected = LongStringsNotEnabledException.class)
+    public void testChangeKeyNoLongStringExcept() {
+        FitsFactory.setLongStringsEnabled(false);
+        HeaderCard c = new HeaderCard("TEST", "abc", "no comment");
+        // No space for ''...
+        c.changeKey("HIERARCH.TEST.AAA.BBB.CCC.DDD.EEE.FFF.GGG.HHH.III.JJJ.KKK.LLL.MMM.NNN.OOO.PP");
+    }
+
+    @Test(expected = LongValueException.class)
+    public void testChangeKeyNumberExcept() {
+        HeaderCard c = new HeaderCard("TEST", "123", "no comment");
+        // No space for ''...
+        c.changeKey("HIERARCH.TEST.AAA.BBB.CCC.DDD.EEE.FFF.GGG.HHH.III.JJJ.KKK.LLL.MMM.NNN.OOO.PPP");
+    }
+
+    @Test(expected = LongValueException.class)
+    public void testNoLongStringExcept() {
+        FitsFactory.setLongStringsEnabled(true);
+        HeaderCard c = new HeaderCard("HIERARCH.TEST.AAA.BBB.CCC.DDD.EEE.FFF.GGG.HHH.III.JJJ.KKK.LLL.MMM.NNN.OOO.PP", "abc",
+                "no comment");
+        FitsFactory.setLongStringsEnabled(false);
+        c.toString();
+    }
+
+    @Test(expected = HeaderCardException.class)
+    public void testCreateInvalidStringExcept() {
+        HeaderCard c = new HeaderCard("TEST", "illegal \r\t in string value", "no comment");
+    }
+
+    @Test
+    public void testHierarchNullString() {
+        HeaderCard c = new HeaderCard("HIERARCH.TEST.AAA.BBB", (String) null, null);
+        Assert.assertEquals("HIERARCH TEST AAA BBB =", c.toString().trim());
+    }
 }
